--------------------------------------------------------------------------------
-- CERN (BE-CO-HT)
-- Top level entity of CONV-TTL-BLO V1
-- http://www.ohwr.org/projects/conv-ttl-blo
--------------------------------------------------------------------------------
--
-- unit name: image1_core.vhd
--
-- author: Carlos Gil Soriano (gilsoriano@gmail.com)
--
-- date: 01-12-2012
--
-- version: 0.9
--
-- description: Core of image1 for CONV-TTL-BLO V2
--
-- dependencies:
--
--------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
--------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--------------------------------------------------------------------------------


library IEEE;
library unisim;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.image1_pkg.ALL;
use work.image1_wrappers_pkg.ALL;
use work.image1_led_pkg.ALL;
use work.wishbone_pkg.ALL;
use work.bicolor_led_ctrl_pkg.ALL;
use UNISIM.VCOMPONENTS.ALL;

entity image1_core is
   generic(g_NUMBER_OF_CHANNELS : NATURAL := 6);
   port (rst_n                : in  STD_LOGIC;
         clk_20MHz_i          : in  STD_LOGIC;
         clk_125MHz_i         : in  STD_LOGIC;
         --! LEDs
         led_array_o          : out t_led_array_o;
         --! I/Os for pulses
         led_front_n          : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         led_rear_n           : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         pulse_i_front_n      : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         pulse_o_front        : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         pulse_i_rear         : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         pulse_o_rear         : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         inv_i_n              : in  STD_LOGIC_VECTOR(4 downto 1);
         inv_o                : out STD_LOGIC_VECTOR(4 downto 1);
         --! Lines for the i2c_slave
         i2c_slave_i          : in  t_i2c_slave_i;
         i2c_slave_o          : out t_i2c_slave_o;
         --! FPGA Geographical address pins (reused for i2c address)
         FPGA_GA              : in  STD_LOGIC_VECTOR(4 downto 0);
         FPGA_GAP             : in  STD_LOGIC;
         --! Pins of the SPI interface to write into the Flash memory
         spi_master_i         : in  t_spi_master_i;
         spi_master_o         : out t_spi_master_o;
         --! RTM identifiers, should match with the expected values
         --! TODO: add matching
         fpga_o_en            : out STD_LOGIC;
         fpga_o_blo_en        : out STD_LOGIC;
         fpga_o_ttl_en        : out STD_LOGIC;
         fpga_o_inv_en        : out STD_LOGIC;
         level                : in  STD_LOGIC;
         switch_i             : in  STD_LOGIC;--! General enable
         manual_rst_n_o       : out STD_LOGIC;--! It allows power sequencing of the
                                              --! 24V rail after a security given
                                              --! delay
         rtm_i                : in  t_rtm_i);
end image1_core;


architecture Behavioral of image1_core is

   type t_clocks is
      record
         PLL_IN      : STD_LOGIC; --! 125MHz  Input clock
         PLL_FB_IN   : STD_LOGIC;
         PLL_FB_OUT  : STD_LOGIC;
         PLL_SYS_A   : STD_LOGIC; --! 50MHz   Wishbone clock
         PLL_SYS_B   : STD_LOGIC; --! 200MHz  Pulse sampling
         SYS_A       : STD_LOGIC;
         SYS_B       : STD_LOGIC;
      end record;

   type t_rst is
      record
         PLL    : STD_LOGIC_VECTOR(c_RST_PLL_CLKS - 1 downto 0);
         SYS_A  : STD_LOGIC_VECTOR(c_RST_A_CLKS   - 1 downto 0);
         SYS_B  : STD_LOGIC_VECTOR(c_RST_B_CLKS   - 1 downto 0);
      end record;

   signal s_clk      : t_clocks;
   signal s_rst      : t_rst := (PLL   => (others => '1'),
                                 SYS_A => (others => '1'),
                                 SYS_B => (others => '1'));

   signal s_rst_n    : STD_LOGIC;
   signal s_locked   : STD_LOGIC;

   signal s_ok_RTMM  : STD_LOGIC;
   signal s_ok_RTMP  : STD_LOGIC;

   signal s_slave_i  : t_wishbone_slave_in_array   (c_NUM_MASTERS - 1 downto 0);
   signal s_slave_o  : t_wishbone_slave_out_array  (c_NUM_MASTERS - 1 downto 0);
   signal s_master_i : t_wishbone_master_in_array  (c_NUM_SLAVES  - 1 downto 0);
   signal s_master_o : t_wishbone_master_out_array (c_NUM_SLAVES  - 1 downto 0);

   signal s_i2c_addr : STD_LOGIC_VECTOR(6 downto 0);

   signal s_pulse_i_front     : STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
   signal s_leds_array_image1 : t_leds_array;
   signal s_led_state_array   : t_led_state_array(
                                    c_NB_ARRAY_LEDS - 1 downto 0);


--   signal s_check_cfg : BOOLEAN;
begin

--   s_check_cfg     <= check_sys_cfg;

      s_i2c_addr      <= f_RENESAS_I2C_ADDRESSING(UNSIGNED(FPGA_GA));

      s_leds_array_image1.top.ERR    <= '1' when s_ok_RTMM = '1' AND s_ok_RTMP = '1'
                                            else '0';

      s_clk.PLL_IN    <= clk_125MHz_i;

      s_pulse_i_front <= not(pulse_i_front_n);

   s_clk_fb_bufg   : BUFG
     port map (O => s_clk.PLL_FB_IN, I => s_clk.PLL_FB_OUT);

   -- set up the fabric PLL_BASE to drive the BUFPLL
   pll_base_inst : PLL_BASE
     generic map (
       BANDWIDTH          => "OPTIMIZED",
       CLK_FEEDBACK       => "CLKFBOUT",
       COMPENSATION       => "SYSTEM_SYNCHRONOUS",
       DIVCLK_DIVIDE      => 1,  CLKFBOUT_MULT  => c_CLKFBOUT_MULT,
                                 CLKFBOUT_PHASE => 0.000,
       CLKOUT0_DIVIDE     => c_CLKOUTA_DIVIDE, CLKOUT0_PHASE      => 0.000,
                                               CLKOUT0_DUTY_CYCLE => 0.500,
       CLKOUT1_DIVIDE     => c_CLKOUTB_DIVIDE, CLKOUT1_PHASE      => 0.000,
                                               CLKOUT1_DUTY_CYCLE => 0.500,
       CLKIN_PERIOD       => c_CLKIN_PERIOD,
       REF_JITTER         => 0.010)
     port map (
       -- Output clocks
       CLKFBOUT => s_clk.PLL_FB_OUT,
       CLKOUT0  => s_clk.PLL_SYS_A,
       CLKOUT1  => s_clk.PLL_SYS_B,
       CLKOUT2  => open,      CLKOUT3  => open,
       CLKOUT4  => open,      CLKOUT5  => open,
       -- Status and control signals
       LOCKED   => s_locked,  RST      => s_rst.PLL(c_RST_PLL_CLKS - 1),
       -- Input clock control
       CLKFBIN  => s_clk.PLL_FB_IN,
       CLKIN    => s_clk.PLL_IN);

   s_clk_50M_bufg : BUFG
     port map (O => s_clk.SYS_A,     I => s_clk.PLL_SYS_A);

   s_clk_200M_bufg : BUFG
     port map (O => s_clk.SYS_B,     I => s_clk.PLL_SYS_B);

   --! @brief            Reset chain for PLL
   --! @param s_clk.PLL  Wishbone clock
   p_reset_PLL_chain : process(s_clk.PLL_IN) is
   begin
      if rising_edge(s_clk.PLL_IN) then
            s_rst.PLL <= (others => '1');

            s_rst.PLL(0) <= '0';
            for i in 1 to c_RST_PLL_CLKS - 1 loop
               s_rst.PLL(i) <= s_rst.PLL(i-1);
            end loop;

            if s_rst.PLL(c_RST_PLL_CLKS - 1) = '1' then
            end if;

         end if;
   end process p_reset_PLL_chain;


   --! @brief              Reset chain
   --! @param s_clk.SYS_A  Wishbone clock
   p_reset_A_chain : process(s_clk.SYS_A) is
   begin
      if rising_edge(s_clk.SYS_A) then
         if s_locked = '0' then
            s_rst.SYS_A    <= (others => '1');
         else
            s_rst.SYS_A(0) <= '0';
            for i in 1 to c_RST_A_CLKS - 1 loop
               s_rst.SYS_A(i) <= s_rst.SYS_A(i-1);
            end loop;

            if s_rst.SYS_A(c_RST_A_CLKS - 1) = '1' then
            end if;

         end if;
      end if;
   end process p_reset_A_chain;

   --! @brief              Reset chain
   --! @param s_clk.SYS_B  System of clock domain B
   p_reset_B_chain : process(s_clk.SYS_B) is
   begin
      if rising_edge(s_clk.SYS_B) then
         if s_locked = '0' then
            s_rst.SYS_B    <= (others => '1');
         else
            s_rst.SYS_B(0) <= '0';
            for i in 1 to c_RST_B_CLKS - 1 loop
               s_rst.SYS_B(i) <= s_rst.SYS_B(i-1);
            end loop;

            if s_rst.SYS_B(c_RST_B_CLKS - 1) = '1' then
            end if;

         end if;
      end if;
   end process p_reset_B_chain;


   inst_basic_trigger: basic_trigger_top
   generic map(g_NUMBER_OF_CHANNELS   => c_NUMBER_OF_CHANNELS,
               g_CLK_PERIOD           => c_CLKB_PERIOD,
               g_OUTPUT_PULSE_LENGTH  => c_OUTPUT_PULSE_LENGTH,
               g_LED_BLINKING_LENGTH  => c_LED_BLINKING_LENGTH)
   port map(clk_i          => s_clk.SYS_B,
            rst_i          => s_rst.SYS_B(c_RST_B_CLKS - 1),
            led_pw_o       => s_leds_array_image1.top.PWR,
            led_err_o      => open,
            led_ttl_o      => s_leds_array_image1.top.TTL_N,
            fpga_o_en      => fpga_o_en,
            fpga_o_ttl_en  => fpga_o_ttl_en,
            fpga_o_inv_en  => fpga_o_inv_en,
            fpga_o_blo_en  => fpga_o_blo_en,
            level          => level,
            switch_i       => switch_i,
            manual_rst_n_o => manual_rst_n_o,
            pulse_i_front  => s_pulse_i_front,
            pulse_o_front  => pulse_o_front,
            pulse_i_rear   => pulse_i_rear,
            pulse_o_rear   => pulse_o_rear,
            led_o_front    => led_front_n,
            led_o_rear     => led_rear_n,
            led_link_up_o  => s_leds_array_image1.middle.WR_LINK,
            led_pps_o      => s_leds_array_image1.middle.WR_GMT,
            led_wr_ok_o    => s_leds_array_image1.middle.WR_OK,
            inv_i          => inv_i_n,
            inv_o          => inv_o);

   --! We are always the slave so we disable writes into SCL pin
   i2c_slave_o.SCL_OE <= '0';

   -- s_slave_o(c_MASTER_I2C_SLAVE).stall;
   -- s_slave_o(c_MASTER_I2C_SLAVE).int;
   -- s_master_i(c_SLAVE_I2C_SLAVE).stall,
   -- s_master_i(c_SLAVE_I2C_SLAVE).int,

   s_slave_i(c_MASTER_I2C_SLAVE).adr(31 downto 16) <= (others => '0');

   inst_i2c_slave: i2c_slave_top
   port map(sda_oen           => i2c_slave_o.SDA_OE,
            sda_i             => i2c_slave_i.SDA_I,
            sda_o             => i2c_slave_o.SDA_O,
            scl_oen           => i2c_slave_o.SCL_OE,
            scl_i             => i2c_slave_i.SCL_I,
            scl_o             => i2c_slave_o.SCL_O,
            wb_clk            => s_clk.SYS_A,
            wb_rst_i          => s_rst.SYS_A(c_RST_A_CLKS - 1),
            wb_master_stb_o   => s_slave_i(c_MASTER_I2C_SLAVE).stb,
            wb_master_cyc_o   => s_slave_i(c_MASTER_I2C_SLAVE).cyc,
            wb_master_sel_o   => s_slave_i(c_MASTER_I2C_SLAVE).sel,
            wb_master_we_o    => s_slave_i(c_MASTER_I2C_SLAVE).we,
            wb_master_data_i  => s_slave_o(c_MASTER_I2C_SLAVE).dat,
            wb_master_data_o  => s_slave_i(c_MASTER_I2C_SLAVE).dat,
            wb_master_addr_o  => s_slave_i(c_MASTER_I2C_SLAVE).adr(15 downto 0),
            wb_master_ack_i   => s_slave_o(c_MASTER_I2C_SLAVE).ack,
            wb_master_rty_i   => s_slave_o(c_MASTER_I2C_SLAVE).rty,
            wb_master_err_i   => s_slave_o(c_MASTER_I2C_SLAVE).err,
            wb_slave_stb_i    => s_master_o(c_SLAVE_I2C_SLAVE).stb,
            wb_slave_cyc_i    => s_master_o(c_SLAVE_I2C_SLAVE).cyc,
            wb_slave_sel_i    => s_master_o(c_SLAVE_I2C_SLAVE).sel,
            wb_slave_we_i     => s_master_o(c_SLAVE_I2C_SLAVE).we,
            wb_slave_data_i   => s_master_o(c_SLAVE_I2C_SLAVE).dat,
            wb_slave_data_o   => s_master_i(c_SLAVE_I2C_SLAVE).dat,
            wb_slave_addr_i   => s_master_o(c_SLAVE_I2C_SLAVE).adr( 5 downto 2),
            wb_slave_ack_o    => s_master_i(c_SLAVE_I2C_SLAVE).ack,
            wb_slave_rty_o    => s_master_i(c_SLAVE_I2C_SLAVE).rty,
            wb_slave_err_o    => s_master_i(c_SLAVE_I2C_SLAVE).err,
            pf_wb_addr_o      => open,
            rd_done_o         => open,
            wr_done_o         => open,
            i2c_addr_i        => s_i2c_addr);


   inst_m25p32: m25p32_top
   port map(wb_rst_i          => s_rst.SYS_A(c_RST_A_CLKS - 1),
            wb_clk            => s_clk.SYS_A,
            wb_we_i           => s_master_o(c_SLAVE_M25P32).we,
            wb_stb_i          => s_master_o(c_SLAVE_M25P32).stb,
            wb_cyc_i          => s_master_o(c_SLAVE_M25P32).cyc,
            wb_sel_i          => s_master_o(c_SLAVE_M25P32).sel,
            wb_data_i         => s_master_o(c_SLAVE_M25P32).dat,
            wb_data_o         => s_master_i(c_SLAVE_M25P32).dat,
            wb_addr_i         => s_master_o(c_SLAVE_M25P32).adr( 8 downto 2),
            wb_ack_o          => s_master_i(c_SLAVE_M25P32).ack,
            wb_rty_o          => s_master_i(c_SLAVE_M25P32).rty,
            wb_err_o          => s_master_i(c_SLAVE_M25P32).err,
            miso_word_rcv     => open,
            op_finished_o     => open,
            prom_mosi_o       => spi_master_o.MOSI,
            prom_cclk_o       => spi_master_o.CCLK,
            prom_cs0_b_n_o    => spi_master_o.CSO_B_N,
            prom_din_i        => spi_master_i.DIN);


   inst_multiboot: multiboot_top
   port map(wb_rst_i  => s_rst.SYS_A(c_RST_A_CLKS - 1),
            wb_clk    => s_clk.SYS_A,
            wb_we_i   => s_master_o(c_SLAVE_MULTIBOOT).we,
            wb_stb_i  => s_master_o(c_SLAVE_MULTIBOOT).stb,
            wb_cyc_i  => s_master_o(c_SLAVE_MULTIBOOT).cyc,
            wb_sel_i  => s_master_o(c_SLAVE_MULTIBOOT).sel,
            wb_data_i => s_master_o(c_SLAVE_MULTIBOOT).dat,
            wb_data_o => s_master_i(c_SLAVE_MULTIBOOT).dat,
            wb_addr_i => s_master_o(c_SLAVE_MULTIBOOT).adr( 5 downto 2),
            wb_ack_o  => s_master_i(c_SLAVE_MULTIBOOT).ack,
            wb_rty_o  => s_master_i(c_SLAVE_MULTIBOOT).rty,
            wb_err_o  => s_master_i(c_SLAVE_MULTIBOOT).err);


      s_master_i(c_SLAVE_I2C_SLAVE).stall  <= '0';
      s_master_i(c_SLAVE_I2C_SLAVE).int    <= '0';
      s_master_i(c_SLAVE_M25P32).stall     <= '0';
      s_master_i(c_SLAVE_M25P32).int       <= '0';
      s_master_i(c_SLAVE_MULTIBOOT).stall  <= '0';
      s_master_i(c_SLAVE_MULTIBOOT).int    <= '0';

   inst_wb_crossbar: xwb_crossbar
   generic map(g_num_masters => c_NUM_MASTERS,
               g_num_slaves  => c_NUM_SLAVES,
               g_registered  => false,
               -- Address of the slaves connected
               --! It should be noted that the default address length is 32
               --! In our project only 16 bits are addressable
               g_address     => c_addresses,
               g_mask        => c_masks)
   port map(clk_sys_i => s_clk.SYS_A,
            rst_n_i   => s_rst.SYS_A(c_RST_A_CLKS - 1),
            slave_i   => s_slave_i ,
            slave_o   => s_slave_o ,
            master_i  => s_master_i,
            master_o  => s_master_o);

   --! Here are organized in the same disposition as in the front panel.
   --! Take a look to image1_led_pkg.vhd for the correct order for
   --! bicolor_led_ctrl
   s_led_state_array(c_LED_NB_PWR)    <= f_LED_STATE(c_LED_COLOR_PWR)
                            when s_leds_array_image1.top.PWR = '1'
                            else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_ERR)    <= f_LED_STATE(c_LED_COLOR_ERR)
                            when s_leds_array_image1.top.ERR = '1'
                            else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_TTL_N)  <= f_LED_STATE(c_LED_COLOR_TTL_N)
                            when s_leds_array_image1.top.TTL_N = '1'
                            else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_I2C)    <= f_LED_STATE(c_LED_COLOR_I2C)
                            when s_leds_array_image1.top.I2C = '1'
                            else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_WR_OK)   <= f_LED_STATE(c_LED_COLOR_WR_OK)
                            when s_leds_array_image1.middle.WR_OK = '1'
                            else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_WR_LINK) <= f_LED_STATE(c_LED_COLOR_WR_LINK)
                            when s_leds_array_image1.middle.WR_LINK = '1'
                            else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_WR_GMT)  <= f_LED_STATE(c_LED_COLOR_WR_GMT)
                            when s_leds_array_image1.middle.WR_GMT = '1'
                            else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_WR_ADDR) <= f_LED_STATE(c_LED_COLOR_WR_ADDR)
                            when s_leds_array_image1.middle.WR_ADDR = '1'
                            else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_MULTICAST3) <= f_LED_STATE(c_LED_COLOR_MULTICAST3)
                            when s_leds_array_image1.bottom.MULTICAST(3) = '1'
                            else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_MULTICAST2) <= f_LED_STATE(c_LED_COLOR_MULTICAST2)
                            when s_leds_array_image1.bottom.MULTICAST(2) = '1'
                            else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_MULTICAST1) <= f_LED_STATE(c_LED_COLOR_MULTICAST1)
                            when s_leds_array_image1.bottom.MULTICAST(1) = '1'
                            else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_MULTICAST0) <= f_LED_STATE(c_LED_COLOR_MULTICAST0)
                            when s_leds_array_image1.bottom.MULTICAST(0) = '1'
                            else f_LED_STATE(c_LED_OFF);

   s_rst_n   <= not(s_rst.SYS_A(c_RST_A_CLKS - 1));

   inst_bicolor_led_ctrl: bicolor_led_ctrl
   generic map(g_NB_COLUMN    => c_NB_COLUMN,
               g_NB_LINE      => c_NB_LINE,
               g_CLK_FREQ     => 20000000,
               g_REFRESH_RATE => 250)
   port map(rst_n_i         => s_rst_n,
            clk_i           => clk_20MHz_i,
            led_intensity_i => c_LED_INTENSITY,
            led_state_i     => f_STD_LOGIC_VECTOR(s_led_state_array),
            column_o(0)     => led_array_o.WR_OWNADDR_I2C,
            column_o(1)     => led_array_o.WR_GMT_TTL_TTLN,
            column_o(2)     => led_array_o.WR_LINK_SYSERROR,
            column_o(3)     => led_array_o.WR_OK_SYSPW,
            column_o(4)     => led_array_o.MULTICAST_2_0,
            column_o(5)     => led_array_o.MULTICAST_3_1,
            line_o(0)       => led_array_o.CTRL0,
            line_o(1)       => led_array_o.CTRL1,
            line_oen_o(0)   => led_array_o.CTRL0_OEN,
            line_oen_o(1)   => led_array_o.CTRL1_OEN);

   inst_rtm_detector: rtm_detector
   port map(RTMM_i     => rtm_i.RTMM_N,
            RTMP_i     => rtm_i.RTMP_N,
            ok_RTMM_o  => s_ok_RTMM,
            ok_RTMP_o  => s_ok_RTMP);


end Behavioral;
