--------------------------------------------------------------------------------
-- CERN (BE-CO-HT)
-- Top level entity of CONV-TTL-BLO V1
-- http://www.ohwr.org/projects/conv-ttl-blo
--------------------------------------------------------------------------------
--
-- unit name: image1_top.vhd
--
-- author: Carlos Gil Soriano (gilsoriano@gmail.com)
--
-- date: 01-12-2012
--
-- version: 0.9
--
-- description: Top entity of CONV-TTL-BLO V1
--
-- dependencies:
--
--------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
--------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--------------------------------------------------------------------------------


library IEEE;
library unisim;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.image1_pkg.ALL;
use work.image1_wrappers_pkg.ALL;
use work.wishbone_pkg.ALL;
use UNISIM.VCOMPONENTS.ALL;

entity image1_top is
   generic(g_NUMBER_OF_CHANNELS : NATURAL := 6);
   port (RST_N                : in  STD_LOGIC;
         CLK20_VCXO           : in  STD_LOGIC;
         FPGA_CLK_P           : in  STD_LOGIC; --Using the 125MHz clock
         FPGA_CLK_N           : in  STD_LOGIC;
         --! LEDs
         LED_CTRL0            : out STD_LOGIC;
         LED_CTRL0_OEN        : out STD_LOGIC;
         LED_CTRL1            : out STD_LOGIC;
         LED_CTRL1_OEN        : out STD_LOGIC;
         LED_MULTICAST_2_0    : out STD_LOGIC;
         LED_MULTICAST_3_1    : out STD_LOGIC;
         LED_WR_GMT_TTL_TTLN  : out STD_LOGIC;
         LED_WR_LINK_SYSERROR : out STD_LOGIC;
         LED_WR_OK_SYSPW      : out STD_LOGIC;
         LED_WR_OWNADDR_I2C   : out STD_LOGIC;
         --! I/Os for pulses
         PULSE_FRONT_LED_N    : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         PULSE_REAR_LED_N     : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         FPGA_INPUT_TTL_N     : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         FPGA_OUT_TTL         : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         FPGA_BLO_IN          : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         FPGA_TRIG_BLO        : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         INV_IN_N             : in  STD_LOGIC_VECTOR(4 downto 1);
         INV_OUT              : out STD_LOGIC_VECTOR(4 downto 1);
         --! Lines for the i2c_slave
         SCL_I                : in  STD_LOGIC;
         SCL_O                : out STD_LOGIC;
         SCL_OE               : out STD_LOGIC;
         SDA_I                : in  STD_LOGIC;
         SDA_O                : out STD_LOGIC;
         SDA_OE               : out STD_LOGIC;
         FPGA_GA              : in  STD_LOGIC_VECTOR(4 downto 0);
         FPGA_GAP             : in  STD_LOGIC;
         --! Pins of the SPI interface to write into the Flash memory
         FPGA_PROM_CCLK       : out STD_LOGIC;
         FPGA_PROM_CSO_B_N    : out STD_LOGIC;
         FPGA_PROM_DIN        : in  STD_LOGIC;
         FPGA_PROM_MOSI       : out STD_LOGIC;
         --! RTM identifiers, should match with the expected values
         --! TODO: add matching
         FPGA_OE              : out STD_LOGIC;
         FPGA_BLO_OE          : out STD_LOGIC;
         FPGA_TRIG_TTL_OE     : out STD_LOGIC;
         FPGA_INV_OE          : out STD_LOGIC;
         LEVEL                : in  STD_LOGIC;--!TTL/INV_TTL_N
         EXTRA_SWITCH         : in  STD_LOGIC_VECTOR(1 downto 1);--! General enable
         MR_N                 : out STD_LOGIC;--! It allows power sequencing of the
                                              --! 24V rail after a security given
                                              --! delay
         FPGA_RTMM_N          : in STD_LOGIC_VECTOR(2 downto 0);
         FPGA_RTMP_N          : in STD_LOGIC_VECTOR(2 downto 0));
end image1_top;


architecture Behavioral of image1_top is

   component image1_core is
      generic(g_NUMBER_OF_CHANNELS : NATURAL := 6);
      port (rst_n           : in  STD_LOGIC;
            clk_20MHz_i     : in  STD_LOGIC;
            clk_125MHz_i    : in  STD_LOGIC;
            --! LEDs
            led_array_o     : out t_led_array_o;
            --! I/Os for pulses
            led_front_n     : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
            led_rear_n      : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
            pulse_i_front_n : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
            pulse_o_front   : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
            pulse_i_rear    : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
            pulse_o_rear    : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
            inv_i_n         : in  STD_LOGIC_VECTOR(4 downto 1);
            inv_o           : out STD_LOGIC_VECTOR(4 downto 1);
            --! Lines for the i2c_slave
            i2c_slave_i     : in  t_i2c_slave_i;
            i2c_slave_o     : out t_i2c_slave_o;
            --! FPGA Geographical address pins (reused for i2c address)
            FPGA_GA         : in  STD_LOGIC_VECTOR(4 downto 0);
            FPGA_GAP        : in  STD_LOGIC;
            --! Pins of the SPI interface to write into the Flash memory
            spi_master_i    : in  t_spi_master_i;
            spi_master_o    : out t_spi_master_o;
            --! RTM identifiers, should match with the expected values
            --! TODO: add matching
            fpga_o_en       : out STD_LOGIC;
            fpga_o_blo_en   : out STD_LOGIC;
            fpga_o_ttl_en   : out STD_LOGIC;
            fpga_o_inv_en   : out STD_LOGIC;
            level           : in  STD_LOGIC;
            switch_i        : in  STD_LOGIC;   --! General enable
            manual_rst_n_o  : out STD_LOGIC;   --! It allows power sequencing of the
                                               --! 24V rail after a security given
                                               --! delay
            rtm_i           : in  t_rtm_i);
   end component;

   signal s_clk_125MHz    : STD_LOGIC;
   signal s_led_array     : t_led_array_o := c_led_array_default;
   signal s_i2c_slave_i   : t_i2c_slave_i;
   signal s_i2c_slave_o   : t_i2c_slave_o;
   signal s_spi_master_i  : t_spi_master_i;
   signal s_spi_master_o  : t_spi_master_o;
   signal s_rtm_i         : t_rtm_i;
   signal s_switch_i      : STD_LOGIC_VECTOR(1 downto 1);

begin

   inst_125m_IBUFGDS : IBUFGDS
   generic map (DIFF_TERM    => TRUE,  IBUF_LOW_PWR => TRUE)
   port map (O  => s_clk_125MHz, I  => FPGA_CLK_P,
                                 IB => FPGA_CLK_N);

      LED_CTRL0              <= s_led_array.CTRL0;
      LED_CTRL0_OEN          <= s_led_array.CTRL0_OEN;
      LED_CTRL1              <= s_led_array.CTRL1;
      LED_CTRL1_OEN          <= s_led_array.CTRL1_OEN;
      LED_MULTICAST_2_0      <= s_led_array.MULTICAST_2_0;
      LED_MULTICAST_3_1      <= s_led_array.MULTICAST_3_1;
      LED_WR_GMT_TTL_TTLN    <= s_led_array.WR_GMT_TTL_TTLN;
      LED_WR_LINK_SYSERROR   <= s_led_array.WR_LINK_SYSERROR;
      LED_WR_OK_SYSPW        <= s_led_array.WR_OK_SYSPW;
      LED_WR_OWNADDR_I2C     <= s_led_array.WR_OWNADDR_I2C;

      s_i2c_slave_i.SCL_I    <= SCL_I;
      s_i2c_slave_i.SDA_I    <= SDA_I;
      SCL_O                  <= s_i2c_slave_o.SCL_O;
      SCL_OE                 <= s_i2c_slave_o.SCL_OE;
      SDA_O                  <= s_i2c_slave_o.SDA_O;
      SDA_OE                 <= s_i2c_slave_o.SDA_OE;

      s_spi_master_i.DIN     <= FPGA_PROM_DIN;
      FPGA_PROM_CCLK         <= s_spi_master_o.CCLK;
      FPGA_PROM_CSO_B_N      <= s_spi_master_o.CSO_B_N;
      FPGA_PROM_MOSI         <= s_spi_master_o.MOSI;

      s_rtm_i.RTMM_N         <= FPGA_RTMM_N;
      s_rtm_i.RTMP_N         <= fpga_rtmp_n;

      s_switch_i             <= EXTRA_SWITCH;

   inst_image1_core: image1_core
      generic map(g_NUMBER_OF_CHANNELS => 6)
      port map(rst_n               => RST_N,
               clk_20MHz_i         => CLK20_VCXO,
               clk_125MHz_i        => s_clk_125MHz,
               led_array_o         => s_led_array,
               led_front_n         => PULSE_FRONT_LED_N,
               led_rear_n          => PULSE_REAR_LED_N,
               pulse_i_front_n     => FPGA_INPUT_TTL_N,
               pulse_o_front       => FPGA_OUT_TTL,
               pulse_i_rear        => FPGA_BLO_IN,
               pulse_o_rear        => FPGA_TRIG_BLO,
               inv_i_n             => INV_IN_N,
               inv_o               => INV_OUT,
               i2c_slave_i         => s_i2c_slave_i,
               i2c_slave_o         => s_i2c_slave_o,
               FPGA_GA             => FPGA_GA ,
               FPGA_GAP            => FPGA_GAP,
               spi_master_i        => s_spi_master_i,
               spi_master_o        => s_spi_master_o,
               fpga_o_en           => FPGA_OE,
               fpga_o_blo_en       => FPGA_BLO_OE,
               fpga_o_ttl_en       => FPGA_TRIG_TTL_OE,
               fpga_o_inv_en       => FPGA_INV_OE,
               level               => LEVEL,
               switch_i            => s_switch_i(1),
               manual_rst_n_o      => MR_N,
               rtm_i               => s_rtm_i);

end Behavioral;
